@@ -302,17 +302,15 @@ namespace Rstats {
     IV is_nan(IV);
   }
 
-  class Vector {
-    public:
+  struct Vector {
     Rstats::VectorType::Enum type;
-    std::map<IV, IV> na_positions;
+    std::map<IV, IV>* na_positions;
     void* values;
-    ~Vector ();
   };
 
   // Macro for Rstats::Vector
 # define RSTATS_DEF_VECTOR_FUNC_UN_IS(FUNC_NAME, ELEMENT_FUNC_NAME) \
-    Rstats::Vector* Rstats::VectorFunc::FUNC_NAME(Rstats::Vector* v1) { \
+    Rstats::Vector* FUNC_NAME(Rstats::Vector* v1) { \
       IV length = Rstats::VectorFunc::get_length(v1); \
       Rstats::Vector* v2 = Rstats::VectorFunc::new_logical(length); \
       Rstats::VectorType::Enum type = Rstats::VectorFunc::get_type(v1); \
@@ -351,7 +349,7 @@ namespace Rstats {
     }
 
 # define RSTATS_DEF_VECTOR_FUNC_UN_MATH(FUNC_NAME, ELEMENT_FUNC_NAME) \
-    Rstats::Vector* Rstats::VectorFunc::FUNC_NAME(Rstats::Vector* v1) { \
+    Rstats::Vector* FUNC_NAME(Rstats::Vector* v1) { \
       IV length = Rstats::VectorFunc::get_length(v1); \
       Rstats::Vector* v2; \
       Rstats::VectorType::Enum type = Rstats::VectorFunc::get_type(v1); \
@@ -384,7 +382,7 @@ namespace Rstats {
     }
     
 # define RSTATS_DEF_VECTOR_FUNC_UN_MATH_INTEGER_TO_DOUBLE(FUNC_NAME, ELEMENT_FUNC_NAME) \
-    Rstats::Vector* Rstats::VectorFunc::FUNC_NAME(Rstats::Vector* v1) { \
+    Rstats::Vector* FUNC_NAME(Rstats::Vector* v1) { \
       IV length = Rstats::VectorFunc::get_length(v1); \
       Rstats::Vector* v2; \
       Rstats::VectorType::Enum type = Rstats::VectorFunc::get_type(v1); \
@@ -417,7 +415,7 @@ namespace Rstats {
     }
 
 # define RSTATS_DEF_VECTOR_FUNC_UN_MATH_COMPLEX_INTEGER_TO_DOUBLE(FUNC_NAME, ELEMENT_FUNC_NAME) \
-    Rstats::Vector* Rstats::VectorFunc::FUNC_NAME(Rstats::Vector* v1) { \
+    Rstats::Vector* FUNC_NAME(Rstats::Vector* v1) { \
       IV length = Rstats::VectorFunc::get_length(v1); \
       Rstats::Vector* v2; \
       Rstats::VectorType::Enum type = Rstats::VectorFunc::get_type(v1); \
@@ -450,7 +448,7 @@ namespace Rstats {
     }
 
 # define RSTATS_DEF_VECTOR_FUNC_BIN_TO_LOGICAL(FUNC_NAME, ELEMENT_FUNC_NAME) \
-    Rstats::Vector* Rstats::VectorFunc::FUNC_NAME(Rstats::Vector* v1, Rstats::Vector* v2) { \
+    Rstats::Vector* FUNC_NAME(Rstats::Vector* v1, Rstats::Vector* v2) { \
       if (Rstats::VectorFunc::get_type(v1) != Rstats::VectorFunc::get_type(v2)) { \
         croak("Can't add different type(Rstats::VectorFunc::%s())", #FUNC_NAME); \
       } \
@@ -500,7 +498,7 @@ namespace Rstats {
     }
     
 # define RSTATS_DEF_VECTOR_FUNC_BIN_MATH(FUNC_NAME, ELEMENT_FUNC_NAME) \
-    Rstats::Vector* Rstats::VectorFunc::FUNC_NAME(Rstats::Vector* v1, Rstats::Vector* v2) { \
+    Rstats::Vector* FUNC_NAME(Rstats::Vector* v1, Rstats::Vector* v2) { \
       if (Rstats::VectorFunc::get_type(v1) != Rstats::VectorFunc::get_type(v2)) { \
         croak("Can't add different type(Rstats::VectorFunc::%s())", #FUNC_NAME); \
       } \
@@ -544,7 +542,7 @@ namespace Rstats {
     }
 
 # define RSTATS_DEF_VECTOR_FUNC_BIN_MATH_INTEGER_TO_DOUBLE(FUNC_NAME, ELEMENT_FUNC_NAME) \
-    Rstats::Vector* Rstats::VectorFunc::FUNC_NAME(Rstats::Vector* v1, Rstats::Vector* v2) { \
+    Rstats::Vector* FUNC_NAME(Rstats::Vector* v1, Rstats::Vector* v2) { \
       if (Rstats::VectorFunc::get_type(v1) != Rstats::VectorFunc::get_type(v2)) { \
         croak("Can't add different type(Rstats::VectorFunc::%s())", #FUNC_NAME); \
       } \
@@ -584,6 +582,8 @@ namespace Rstats {
 
   // Rstats::VectorFunc
   namespace VectorFunc {
+    Rstats::Vector* new_vector();
+    void delete_vector(Rstats::Vector*);
     SV* get_value(Rstats::Vector*, IV);
     SV* get_values(Rstats::Vector*);
     
@@ -603,7 +603,7 @@ namespace Rstats {
     void add_na_position(Rstats::Vector*, IV);
     bool exists_na_position(Rstats::Vector*, IV position);
     void merge_na_positions(Rstats::Vector*, Rstats::Vector*);
-    std::map<IV, IV> get_na_positions(Rstats::Vector*);
+    std::map<IV, IV>* get_na_positions(Rstats::Vector*);
     IV get_length (Rstats::Vector*);
     
     Rstats::Vector* new_character(IV, SV*);
