@@ -238,7 +238,7 @@ SV* Rstats::Util::looks_like_na (SV* sv_value) {
   else {
     SV* sv_na = Rstats::pl_new_sv_pv("NA");
     if (sv_cmp(sv_value, sv_na) == 0) {
-      sv_ret = Rstats::pl_to_perl_obj(Rstats::Vector::new_na(), "Rstats::Vector");
+      sv_ret = Rstats::pl_to_perl_obj(Rstats::VectorFunc::new_na(), "Rstats::Vector");
     }
     else {
       sv_ret = &PL_sv_undef;
