@@ -1,55 +1,55 @@
 #include "Rstats.h"
 
-RSTATS_DEF_VECTOR_FUNC_UN_IS(is_infinite, Rstats::ElementFunc::is_infinite)
-RSTATS_DEF_VECTOR_FUNC_UN_IS(is_finite, Rstats::ElementFunc::is_finite)
-RSTATS_DEF_VECTOR_FUNC_UN_IS(is_nan, Rstats::ElementFunc::is_nan)
+RSTATS_DEF_VECTOR_FUNC_UN_IS(Rstats::VectorFunc::is_infinite, Rstats::ElementFunc::is_infinite)
+RSTATS_DEF_VECTOR_FUNC_UN_IS(Rstats::VectorFunc::is_finite, Rstats::ElementFunc::is_finite)
+RSTATS_DEF_VECTOR_FUNC_UN_IS(Rstats::VectorFunc::is_nan, Rstats::ElementFunc::is_nan)
 
-RSTATS_DEF_VECTOR_FUNC_UN_MATH(negation, Rstats::ElementFunc::negation)
+RSTATS_DEF_VECTOR_FUNC_UN_MATH(Rstats::VectorFunc::negation, Rstats::ElementFunc::negation)
 
-RSTATS_DEF_VECTOR_FUNC_UN_MATH_INTEGER_TO_DOUBLE(sin, Rstats::ElementFunc::sin)
-RSTATS_DEF_VECTOR_FUNC_UN_MATH_INTEGER_TO_DOUBLE(cos, Rstats::ElementFunc::cos)
-RSTATS_DEF_VECTOR_FUNC_UN_MATH_INTEGER_TO_DOUBLE(tan, Rstats::ElementFunc::tan)
-RSTATS_DEF_VECTOR_FUNC_UN_MATH_INTEGER_TO_DOUBLE(sinh, Rstats::ElementFunc::sinh)
-RSTATS_DEF_VECTOR_FUNC_UN_MATH_INTEGER_TO_DOUBLE(cosh, Rstats::ElementFunc::cosh)
-RSTATS_DEF_VECTOR_FUNC_UN_MATH_INTEGER_TO_DOUBLE(tanh, Rstats::ElementFunc::tanh)
-RSTATS_DEF_VECTOR_FUNC_UN_MATH_INTEGER_TO_DOUBLE(log, Rstats::ElementFunc::log)
-RSTATS_DEF_VECTOR_FUNC_UN_MATH_INTEGER_TO_DOUBLE(logb, Rstats::ElementFunc::logb)
-RSTATS_DEF_VECTOR_FUNC_UN_MATH_INTEGER_TO_DOUBLE(log10, Rstats::ElementFunc::log10)
-RSTATS_DEF_VECTOR_FUNC_UN_MATH_INTEGER_TO_DOUBLE(log2, Rstats::ElementFunc::log2)
-RSTATS_DEF_VECTOR_FUNC_UN_MATH_INTEGER_TO_DOUBLE(expm1, Rstats::ElementFunc::expm1)
-RSTATS_DEF_VECTOR_FUNC_UN_MATH_INTEGER_TO_DOUBLE(exp, Rstats::ElementFunc::exp)
-RSTATS_DEF_VECTOR_FUNC_UN_MATH_INTEGER_TO_DOUBLE(sqrt, Rstats::ElementFunc::sqrt)
-RSTATS_DEF_VECTOR_FUNC_UN_MATH_INTEGER_TO_DOUBLE(atan, Rstats::ElementFunc::atan)
-RSTATS_DEF_VECTOR_FUNC_UN_MATH_INTEGER_TO_DOUBLE(asin, Rstats::ElementFunc::asin)
-RSTATS_DEF_VECTOR_FUNC_UN_MATH_INTEGER_TO_DOUBLE(acos, Rstats::ElementFunc::acos)
-RSTATS_DEF_VECTOR_FUNC_UN_MATH_INTEGER_TO_DOUBLE(asinh, Rstats::ElementFunc::asinh)
-RSTATS_DEF_VECTOR_FUNC_UN_MATH_INTEGER_TO_DOUBLE(acosh, Rstats::ElementFunc::acosh)
-RSTATS_DEF_VECTOR_FUNC_UN_MATH_INTEGER_TO_DOUBLE(atanh, Rstats::ElementFunc::atanh)
-RSTATS_DEF_VECTOR_FUNC_UN_MATH_INTEGER_TO_DOUBLE(Conj, Rstats::ElementFunc::Conj)
+RSTATS_DEF_VECTOR_FUNC_UN_MATH_INTEGER_TO_DOUBLE(Rstats::VectorFunc::sin, Rstats::ElementFunc::sin)
+RSTATS_DEF_VECTOR_FUNC_UN_MATH_INTEGER_TO_DOUBLE(Rstats::VectorFunc::cos, Rstats::ElementFunc::cos)
+RSTATS_DEF_VECTOR_FUNC_UN_MATH_INTEGER_TO_DOUBLE(Rstats::VectorFunc::tan, Rstats::ElementFunc::tan)
+RSTATS_DEF_VECTOR_FUNC_UN_MATH_INTEGER_TO_DOUBLE(Rstats::VectorFunc::sinh, Rstats::ElementFunc::sinh)
+RSTATS_DEF_VECTOR_FUNC_UN_MATH_INTEGER_TO_DOUBLE(Rstats::VectorFunc::cosh, Rstats::ElementFunc::cosh)
+RSTATS_DEF_VECTOR_FUNC_UN_MATH_INTEGER_TO_DOUBLE(Rstats::VectorFunc::tanh, Rstats::ElementFunc::tanh)
+RSTATS_DEF_VECTOR_FUNC_UN_MATH_INTEGER_TO_DOUBLE(Rstats::VectorFunc::log, Rstats::ElementFunc::log)
+RSTATS_DEF_VECTOR_FUNC_UN_MATH_INTEGER_TO_DOUBLE(Rstats::VectorFunc::logb, Rstats::ElementFunc::logb)
+RSTATS_DEF_VECTOR_FUNC_UN_MATH_INTEGER_TO_DOUBLE(Rstats::VectorFunc::log10, Rstats::ElementFunc::log10)
+RSTATS_DEF_VECTOR_FUNC_UN_MATH_INTEGER_TO_DOUBLE(Rstats::VectorFunc::log2, Rstats::ElementFunc::log2)
+RSTATS_DEF_VECTOR_FUNC_UN_MATH_INTEGER_TO_DOUBLE(Rstats::VectorFunc::expm1, Rstats::ElementFunc::expm1)
+RSTATS_DEF_VECTOR_FUNC_UN_MATH_INTEGER_TO_DOUBLE(Rstats::VectorFunc::exp, Rstats::ElementFunc::exp)
+RSTATS_DEF_VECTOR_FUNC_UN_MATH_INTEGER_TO_DOUBLE(Rstats::VectorFunc::sqrt, Rstats::ElementFunc::sqrt)
+RSTATS_DEF_VECTOR_FUNC_UN_MATH_INTEGER_TO_DOUBLE(Rstats::VectorFunc::atan, Rstats::ElementFunc::atan)
+RSTATS_DEF_VECTOR_FUNC_UN_MATH_INTEGER_TO_DOUBLE(Rstats::VectorFunc::asin, Rstats::ElementFunc::asin)
+RSTATS_DEF_VECTOR_FUNC_UN_MATH_INTEGER_TO_DOUBLE(Rstats::VectorFunc::acos, Rstats::ElementFunc::acos)
+RSTATS_DEF_VECTOR_FUNC_UN_MATH_INTEGER_TO_DOUBLE(Rstats::VectorFunc::asinh, Rstats::ElementFunc::asinh)
+RSTATS_DEF_VECTOR_FUNC_UN_MATH_INTEGER_TO_DOUBLE(Rstats::VectorFunc::acosh, Rstats::ElementFunc::acosh)
+RSTATS_DEF_VECTOR_FUNC_UN_MATH_INTEGER_TO_DOUBLE(Rstats::VectorFunc::atanh, Rstats::ElementFunc::atanh)
+RSTATS_DEF_VECTOR_FUNC_UN_MATH_INTEGER_TO_DOUBLE(Rstats::VectorFunc::Conj, Rstats::ElementFunc::Conj)
 
-RSTATS_DEF_VECTOR_FUNC_UN_MATH_COMPLEX_INTEGER_TO_DOUBLE(Arg, Rstats::ElementFunc::Arg)
-RSTATS_DEF_VECTOR_FUNC_UN_MATH_COMPLEX_INTEGER_TO_DOUBLE(abs, Rstats::ElementFunc::abs)
-RSTATS_DEF_VECTOR_FUNC_UN_MATH_COMPLEX_INTEGER_TO_DOUBLE(Mod, Rstats::ElementFunc::Mod)
-RSTATS_DEF_VECTOR_FUNC_UN_MATH_COMPLEX_INTEGER_TO_DOUBLE(Re, Rstats::ElementFunc::Re)
-RSTATS_DEF_VECTOR_FUNC_UN_MATH_COMPLEX_INTEGER_TO_DOUBLE(Im, Rstats::ElementFunc::Im)
+RSTATS_DEF_VECTOR_FUNC_UN_MATH_COMPLEX_INTEGER_TO_DOUBLE(Rstats::VectorFunc::Arg, Rstats::ElementFunc::Arg)
+RSTATS_DEF_VECTOR_FUNC_UN_MATH_COMPLEX_INTEGER_TO_DOUBLE(Rstats::VectorFunc::abs, Rstats::ElementFunc::abs)
+RSTATS_DEF_VECTOR_FUNC_UN_MATH_COMPLEX_INTEGER_TO_DOUBLE(Rstats::VectorFunc::Mod, Rstats::ElementFunc::Mod)
+RSTATS_DEF_VECTOR_FUNC_UN_MATH_COMPLEX_INTEGER_TO_DOUBLE(Rstats::VectorFunc::Re, Rstats::ElementFunc::Re)
+RSTATS_DEF_VECTOR_FUNC_UN_MATH_COMPLEX_INTEGER_TO_DOUBLE(Rstats::VectorFunc::Im, Rstats::ElementFunc::Im)
 
-RSTATS_DEF_VECTOR_FUNC_BIN_TO_LOGICAL(equal, Rstats::ElementFunc::equal);
-RSTATS_DEF_VECTOR_FUNC_BIN_TO_LOGICAL(not_equal, Rstats::ElementFunc::not_equal);
-RSTATS_DEF_VECTOR_FUNC_BIN_TO_LOGICAL(more_than, Rstats::ElementFunc::more_than);
-RSTATS_DEF_VECTOR_FUNC_BIN_TO_LOGICAL(less_than, Rstats::ElementFunc::less_than);
-RSTATS_DEF_VECTOR_FUNC_BIN_TO_LOGICAL(more_than_or_equal, Rstats::ElementFunc::more_than_or_equal);
-RSTATS_DEF_VECTOR_FUNC_BIN_TO_LOGICAL(less_than_or_equal, Rstats::ElementFunc::less_than_or_equal);
-RSTATS_DEF_VECTOR_FUNC_BIN_TO_LOGICAL(And, Rstats::ElementFunc::And);
-RSTATS_DEF_VECTOR_FUNC_BIN_TO_LOGICAL(Or, Rstats::ElementFunc::Or);
+RSTATS_DEF_VECTOR_FUNC_BIN_TO_LOGICAL(Rstats::VectorFunc::equal, Rstats::ElementFunc::equal);
+RSTATS_DEF_VECTOR_FUNC_BIN_TO_LOGICAL(Rstats::VectorFunc::not_equal, Rstats::ElementFunc::not_equal);
+RSTATS_DEF_VECTOR_FUNC_BIN_TO_LOGICAL(Rstats::VectorFunc::more_than, Rstats::ElementFunc::more_than);
+RSTATS_DEF_VECTOR_FUNC_BIN_TO_LOGICAL(Rstats::VectorFunc::less_than, Rstats::ElementFunc::less_than);
+RSTATS_DEF_VECTOR_FUNC_BIN_TO_LOGICAL(Rstats::VectorFunc::more_than_or_equal, Rstats::ElementFunc::more_than_or_equal);
+RSTATS_DEF_VECTOR_FUNC_BIN_TO_LOGICAL(Rstats::VectorFunc::less_than_or_equal, Rstats::ElementFunc::less_than_or_equal);
+RSTATS_DEF_VECTOR_FUNC_BIN_TO_LOGICAL(Rstats::VectorFunc::And, Rstats::ElementFunc::And);
+RSTATS_DEF_VECTOR_FUNC_BIN_TO_LOGICAL(Rstats::VectorFunc::Or, Rstats::ElementFunc::Or);
 
-RSTATS_DEF_VECTOR_FUNC_BIN_MATH(add, Rstats::ElementFunc::add)
-RSTATS_DEF_VECTOR_FUNC_BIN_MATH(subtract, Rstats::ElementFunc::subtract)
-RSTATS_DEF_VECTOR_FUNC_BIN_MATH(multiply, Rstats::ElementFunc::multiply)
-RSTATS_DEF_VECTOR_FUNC_BIN_MATH(reminder, Rstats::ElementFunc::reminder)
+RSTATS_DEF_VECTOR_FUNC_BIN_MATH(Rstats::VectorFunc::add, Rstats::ElementFunc::add)
+RSTATS_DEF_VECTOR_FUNC_BIN_MATH(Rstats::VectorFunc::subtract, Rstats::ElementFunc::subtract)
+RSTATS_DEF_VECTOR_FUNC_BIN_MATH(Rstats::VectorFunc::multiply, Rstats::ElementFunc::multiply)
+RSTATS_DEF_VECTOR_FUNC_BIN_MATH(Rstats::VectorFunc::reminder, Rstats::ElementFunc::reminder)
 
-RSTATS_DEF_VECTOR_FUNC_BIN_MATH_INTEGER_TO_DOUBLE(divide, Rstats::ElementFunc::divide)
-RSTATS_DEF_VECTOR_FUNC_BIN_MATH_INTEGER_TO_DOUBLE(atan2, Rstats::ElementFunc::atan2)
-RSTATS_DEF_VECTOR_FUNC_BIN_MATH_INTEGER_TO_DOUBLE(pow, Rstats::ElementFunc::pow)
+RSTATS_DEF_VECTOR_FUNC_BIN_MATH_INTEGER_TO_DOUBLE(Rstats::VectorFunc::divide, Rstats::ElementFunc::divide)
+RSTATS_DEF_VECTOR_FUNC_BIN_MATH_INTEGER_TO_DOUBLE(Rstats::VectorFunc::atan2, Rstats::ElementFunc::atan2)
+RSTATS_DEF_VECTOR_FUNC_BIN_MATH_INTEGER_TO_DOUBLE(Rstats::VectorFunc::pow, Rstats::ElementFunc::pow)
 
 
 SV* Rstats::VectorFunc::get_values(Rstats::Vector* v1) {
@@ -104,25 +104,25 @@ std::vector<IV>* Rstats::VectorFunc::get_integer_values(Rstats::Vector* v1) {
 }
 
 Rstats::VectorType::Enum Rstats::VectorFunc::get_type(Rstats::Vector* v1) {
-  return (*v1).type;
+  return v1->type;
 }
 
 void Rstats::VectorFunc::add_na_position(Rstats::Vector* v1, IV position) {
-  (*v1).na_positions[position] = 1;
+  (*v1->na_positions)[position] = 1;
 }
 
 bool Rstats::VectorFunc::exists_na_position(Rstats::Vector* v1, IV position) {
-  return (*v1).na_positions.count(position);
+  return v1->na_positions->count(position);
 }
 
 void Rstats::VectorFunc::merge_na_positions(Rstats::Vector* v2, Rstats::Vector* v1) {
-  for(std::map<IV, IV>::iterator it = (*v1).na_positions.begin(); it != (*v1).na_positions.end(); ++it) {
+  for(std::map<IV, IV>::iterator it = v1->na_positions->begin(); it != v1->na_positions->end(); ++it) {
     Rstats::VectorFunc::add_na_position(v2, it->first);
   }
 }
 
-std::map<IV, IV> Rstats::VectorFunc::get_na_positions(Rstats::Vector* v1) {
-  return (*v1).na_positions;
+std::map<IV, IV>* Rstats::VectorFunc::get_na_positions(Rstats::Vector* v1) {
+  return v1->na_positions;
 }
 
 IV Rstats::VectorFunc::get_length (Rstats::Vector* v1) {
@@ -148,22 +148,62 @@ IV Rstats::VectorFunc::get_length (Rstats::Vector* v1) {
   }
 }
 
+Rstats::Vector* Rstats::VectorFunc::new_vector() {
+  Rstats::Vector* v1 = new Rstats::Vector;
+  v1->na_positions = new std::map<IV, IV>;
+  
+  return v1;
+}
+
+void Rstats::VectorFunc::delete_vector (Rstats::Vector* v1) {
+  IV length = Rstats::VectorFunc::get_length(v1);
+  
+  Rstats::VectorType::Enum type = Rstats::VectorFunc::get_type(v1);
+  switch (type) {
+    case Rstats::VectorType::CHARACTER : {
+      std::vector<SV*>* values = Rstats::VectorFunc::get_character_values(v1);
+      for (IV i = 0; i < length; i++) {
+        if ((*values)[i] != NULL) {
+          SvREFCNT_dec((*values)[i]);
+        }
+      }
+      delete values;
+      break;
+    }
+    case Rstats::VectorType::COMPLEX : {
+      std::vector<std::complex<NV> >* values = Rstats::VectorFunc::get_complex_values(v1);
+      delete values;
+      break;
+    }
+    case Rstats::VectorType::DOUBLE : {
+      std::vector<NV>* values = Rstats::VectorFunc::get_double_values(v1);
+      delete values;
+      break;
+    }
+    case Rstats::VectorType::INTEGER :
+    case Rstats::VectorType::LOGICAL : {
+      std::vector<IV>* values = Rstats::VectorFunc::get_integer_values(v1);
+      delete values;
+    }
+  }
+  delete v1->na_positions;
+}
+
+Rstats::Vector* Rstats::VectorFunc::new_character(IV length) {
+
+  Rstats::Vector* v1 = new_vector();
+  v1->values = new std::vector<SV*>(length);
+  v1->type = Rstats::VectorType::CHARACTER;
+  
+  return v1;
+}
+
 Rstats::Vector* Rstats::VectorFunc::new_character(IV length, SV* sv_str) {
 
   Rstats::Vector* v1 = Rstats::VectorFunc::new_character(length);
   for (IV i = 0; i < length; i++) {
     Rstats::VectorFunc::set_character_value(v1, i, sv_str);
   }
-  v1->type = Rstats::VectorType::CHARACTER;
-  
-  return v1;
-}
-
-Rstats::Vector* Rstats::VectorFunc::new_character(IV length) {
-
-  Rstats::Vector* v1 = new Rstats::Vector;
-  v1->values = new std::vector<SV*>(length);
-  v1->type = Rstats::VectorType::CHARACTER;
   
   return v1;
 }
@@ -189,7 +229,7 @@ void Rstats::VectorFunc::set_character_value(Rstats::Vector* v1, IV pos, SV* val
 
 Rstats::Vector* Rstats::VectorFunc::new_complex(IV length) {
   
-  Rstats::Vector* v1 = new Rstats::Vector;
+  Rstats::Vector* v1 = new_vector();
   v1->values = new std::vector<std::complex<NV> >(length, std::complex<NV>(0, 0));
   v1->type = Rstats::VectorType::COMPLEX;
   
@@ -198,10 +238,10 @@ Rstats::Vector* Rstats::VectorFunc::new_complex(IV length) {
     
 Rstats::Vector* Rstats::VectorFunc::new_complex(IV length, std::complex<NV> z) {
   
-  Rstats::Vector* v1 = new Rstats::Vector;
-  v1->values = new std::vector<std::complex<NV> >(length, z);
-  v1->type = Rstats::VectorType::COMPLEX;
-  
+  Rstats::Vector* v1 = new_complex(length);
+  for (IV i = 0; i < length; i++) {
+    Rstats::VectorFunc::set_complex_value(v1, i, z);
+  }
   return v1;
 }
 
@@ -214,7 +254,7 @@ void Rstats::VectorFunc::set_complex_value(Rstats::Vector* v1, IV pos, std::comp
 }
 
 Rstats::Vector* Rstats::VectorFunc::new_double(IV length) {
-  Rstats::Vector* v1 = new Rstats::Vector;
+  Rstats::Vector* v1 = new_vector();
   v1->values = new std::vector<NV>(length);
   v1->type = Rstats::VectorType::DOUBLE;
   
@@ -222,10 +262,10 @@ Rstats::Vector* Rstats::VectorFunc::new_double(IV length) {
 }
 
 Rstats::Vector* Rstats::VectorFunc::new_double(IV length, NV value) {
-  Rstats::Vector* v1 = new Rstats::Vector;
-  v1->values = new std::vector<NV>(length, value);
-  v1->type = Rstats::VectorType::DOUBLE;
-  
+  Rstats::Vector* v1 = new_double(length);
+  for (IV i = 0; i < length; i++) {
+    Rstats::VectorFunc::set_double_value(v1, i, value);
+  }
   return v1;
 }
 
@@ -239,7 +279,7 @@ void Rstats::VectorFunc::set_double_value(Rstats::Vector* v1, IV pos, NV value)
 
 Rstats::Vector* Rstats::VectorFunc::new_integer(IV length) {
   
-  Rstats::Vector* v1 = new Rstats::Vector;
+  Rstats::Vector* v1 = new_vector();
   v1->values = new std::vector<IV>(length);
   v1->type = Rstats::VectorType::INTEGER;
   
@@ -248,10 +288,10 @@ Rstats::Vector* Rstats::VectorFunc::new_integer(IV length) {
 
 Rstats::Vector* Rstats::VectorFunc::new_integer(IV length, IV value) {
   
-  Rstats::Vector* v1 = new Rstats::Vector;
-  v1->values = new std::vector<IV>(length, value);
-  v1->type = Rstats::VectorType::INTEGER;
-  
+  Rstats::Vector* v1 = new_integer(length);;
+  for (IV i = 0; i < length; i++) {
+    Rstats::VectorFunc::set_integer_value(v1, i, value);
+  }
   return v1;
 }
 
@@ -264,7 +304,8 @@ void Rstats::VectorFunc::set_integer_value(Rstats::Vector* v1, IV pos, IV value)
 }
 
 Rstats::Vector* Rstats::VectorFunc::new_logical(IV length) {
-  Rstats::Vector* v1 = new Rstats::Vector;
+  Rstats::Vector* v1 = new_vector();
+
   v1->values = new std::vector<IV>(length);
   v1->type = Rstats::VectorType::LOGICAL;
   
@@ -272,10 +313,10 @@ Rstats::Vector* Rstats::VectorFunc::new_logical(IV length) {
 }
 
 Rstats::Vector* Rstats::VectorFunc::new_logical(IV length, IV value) {
-  Rstats::Vector* v1 = new Rstats::Vector;
-  v1->values = new std::vector<IV>(length, value);
-  v1->type = Rstats::VectorType::LOGICAL;
-  
+  Rstats::Vector* v1 = new_logical(length);
+  for (IV i = 0; i < length; i++) {
+    Rstats::VectorFunc::set_integer_value(v1, i, value);
+  }
   return v1;
 }
 
@@ -300,16 +341,13 @@ Rstats::Vector* Rstats::VectorFunc::new_inf() {
 }
 
 Rstats::Vector* Rstats::VectorFunc::new_na() {
-  Rstats::Vector* v1 = new Rstats::Vector;
-  v1->values = new std::vector<IV>(1, 0);
-  v1->type = Rstats::VectorType::LOGICAL;
+  Rstats::Vector* v1 = new_logical(1);
   Rstats::VectorFunc::add_na_position(v1, 0);
-  
   return v1;
 }
 
 Rstats::Vector* Rstats::VectorFunc::new_null() {
-  Rstats::Vector* v1 = new Rstats::Vector;
+  Rstats::Vector* v1 = new_vector();
   v1->values = NULL;
   v1->type = Rstats::VectorType::LOGICAL;
   return v1;
